#
# Cookbook Name:: raid
# Recipe:: default
#
# Copyright 2016, maxn
#
# All rights reserved - Do Not Redistribute
#


include_recipe "mdadm"
include_recipe "reboot-handler"
case node["platform"]
	when 'redhat' , 'centos', 'amazon', 'scientific', 'oracle'
		%w{epel-release nano wget mdadm gdisk}.each do |pkg|
			package pkg do
				action :install
			end
		end
		execute "swapoff" do
			command "swapoff -a"
		end
		if node["platform_version"] >= "7" 
			raid_centos7 "/dev/sdb" do
				boot_efi_size "4M"
				boot_size node['part_sizes']['boot_size']
				root_size node['part_sizes']['root_size']
				swap_size node['part_sizes']['swap_size']
				action :mkpart
			end
			mdadm '/dev/md0' do
				devices [ 'missing', '/dev/sdb2' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '0.90'
				action :create
			end
			mdadm '/dev/md1' do
				devices [ 'missing', '/dev/sdb3' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '1.2'
				action :create
			end
			mdadm '/dev/md2' do
				devices [ 'missing', '/dev/sdb5' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '1.2'
				action :create
			end
			raid_part "/dev/md0" do
				file_system "ext2"
				action :mkfs
			end
			raid_part "/dev/md1" do
				file_system "ext4"
				action :mkfs
			end
			raid_part "/dev/md2" do
				file_system "ext4"
				action :mkfs
			end

			raid_part "/dev/sdb4" do
				action :mkswap
			end
			raid_part "/dev/md1" do
				action :mount
			end
			raid_part "/dev/md0" do
				type "boot"
				action :mount
			end
			raid_part "/dev/md2" do
				type "home"
				action :mount
			end
			template "/etc/fstab" do
				source "fstab.erb"
				owner "root"
				group "root"
				mode 0644
			end
			template "/etc/mdadm.conf" do
				source "mdadm.conf.erb"
				owner "root"
				group "root"
				mode 0644
			end
			template "/etc/default/grub" do
				source "grub.erb"
				owner "root"
				group "root"
				mode 0644
			end
			execute "make mdadm" do
				command "mdadm --detail --scan >> /etc/mdadm.conf"
			end	

			raid_centos7 "all" do
				action :mkinit
			end
			execute "grub-set-up" do
				command "grub2-mkconfig -o /boot/grub2/grub.cfg"
			end
			execute "change root in grub" do
				command 'grubby --update-kernel=ALL --args="root=/dev/md1"'
			end
			execute "install grub2" do
				command 'grub2-install /dev/sdb'
			end
		else
			raid_centos6 "/dev/sdb" do
				boot_size node['part_sizes']['boot_size']
				root_size node['part_sizes']['root_size']
				swap_size node['part_sizes']['swap_size']
				action :mkpart
			end
			mdadm '/dev/md0' do
				devices [ 'missing', '/dev/sdb1' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '0.90'
				action :create
			end
			mdadm '/dev/md1' do
				devices [ 'missing', '/dev/sdb2' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '1.2'
				action :create
			end
			mdadm '/dev/md2' do
				devices [ 'missing', '/dev/sdb4' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '1.2'
				action :create
			end
			raid_part "/dev/md0" do
				file_system "ext2"
				action :mkfs
			end
			raid_part "/dev/md1" do
				file_system "ext4"
				action :mkfs
			end
			raid_part "/dev/md2" do
				file_system "ext4"
				action :mkfs
			end

			raid_part "/dev/sdb3" do
				action :mkswap
			end
			raid_part "/dev/md1" do
				action :mount
			end
			raid_part "/dev/md0" do
				type "boot"
				action :mount
			end
			raid_part "/dev/md2" do
				type "home"
				action :mount
			end
			template "/etc/fstab" do
				source "fstab.erb"
				owner "root"
				group "root"
				mode 0644
			end
			template "/etc/mdadm.conf" do
				source "mdadm.conf.erb"
				owner "root"
				group "root"
				mode 0644
			end
			execute "make mdadm.conf" do
				command "mdadm --detail --scan >> /etc/mdadm.conf"
			end	

			template "/root/mkinit" do
				source "mkinitcentos6.erb"
				owner "root"
				group "root"
				mode '1774'
			end
			raid_centos6 "mkinit.." do
				action :mkinit	
			end

			execute "remove devices map" do
				command "rm -rf /boot/grub/device.map /boot/grub2/device.map"
			end
			execute "grubby change args in grub.cfg" do
				command 'grubby --update-kernel=ALL --args="root=/dev/md1 selinux=0" --remove-args="rd_LVM_LV rd_NO_DM rd_NO_MD rhgb quiet rd_NO_LUKS rd_LVM_LV rd_NO_LVM" '
			end
			execute "install grub" do
				command 'echo -e "device (hd0) /dev/sdb \n root (hd0,0) \n setup (hd0)" | grub'
			end
		end
	when 'debian' , 'ubuntu'
		%w{nano wget mdadm gdisk}.each do |pkg|
			package pkg do
				action :install
			end
		end
		execute "swapoff" do
			command "swapoff -a"
		end
			raid_debuntu "/dev/sdb" do
				boot_efi_size "4M"
				boot_size node['part_sizes']['boot_size']
				root_size node['part_sizes']['root_size']
				swap_size node['part_sizes']['swap_size']
				action :mkpart
			end
			mdadm '/dev/md0' do
				devices [ 'missing', '/dev/sdb2' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '0.90'
				action :create
			end
			mdadm '/dev/md1' do
				devices [ 'missing', '/dev/sdb3' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '1.2'
				action :create
			end
			mdadm '/dev/md2' do
				devices [ 'missing', '/dev/sdb5' ]
				level 1
				bitmap "internal --bitmap-chunk=131072"
				metadata '1.2'
				action :create
			end
			raid_part "/dev/md0" do
				file_system "ext2"
				action :mkfs
			end
			raid_part "/dev/md1" do
				file_system "ext4"
				action :mkfs
			end
			raid_part "/dev/md2" do
				file_system "ext4"
				action :mkfs
			end

			raid_part "/dev/sdb4" do
				action :mkswap
			end
			raid_part "/dev/md1" do
				action :mount
			end
			raid_part "/dev/md0" do
				type "boot"
				action :mount
			end
			raid_part "/dev/md2" do
				type "home"
				action :mount
			end
			template "/etc/fstab" do
				source "fstab.erb"
				owner "root"
				group "root"
				mode 0644
			end
			template "/etc/mdadm.conf" do
				source "mdadm.conf.erb"
				owner "root"
				group "root"
				mode 0644
			end
			template "/etc/mdadm/mdadm.conf" do
				source "mdadm.conf.erb"
				owner "root"
				group "root"
				mode 0644
			end
			execute "add args" do
				command 'sed -i -e "s/quiet/rd.auto rd.auto=1 selinux=0 quiet/g" /etc/default/grub'
			end

			execute "diable UUIDS" do
				command 'echo "GRUB_DISABLE_LINUX_UUID=\"true\"" >> /etc/default/grub'
			end
			execute "add preload modules" do
				command 'echo "GRUB_PRELOAD_MODULES=\"part_gpt raid mdraid mdraid09 mdraid1x \"" >> /etc/default/grub'
			end

			execute "make mdadm" do
				command "mdadm --detail --scan >> /etc/mdadm.conf"
			end	

			execute "and make mdadm again " do
				command "mdadm --detail --scan >> /etc/mdadm/mdadm.conf"
			end	

			raid_debuntu "all" do
				action :mkinit
			end
			execute "grub-set-up" do
				command "grub-mkconfig -o /boot/grub/grub.cfg"
			end
			execute "change root in grub" do
				command 'sed -i -e "s#root=.*#root=/dev/md1 rd.auto rd.auto=1 selinux=0 rootdelay=10 quiet#g" /boot/grub/grub.cfg'
			end
			execute "install grub" do
				command 'grub-install /dev/sdb'
			end

end

execute "copy root" do
	command "cp -dpRx / /mnt"
end
execute "copy boot" do
	command "cp -dpRx /boot /mnt"
end



node.run_state['reboot'] = true
reboot 'app_requires_reboot' do
	action :request_reboot
	reason 'Need to reboot when the run completes successfully.'
	delay_mins 0
end
